function [IRF, flag] = CompIRF_ir(paramvec, hh)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Title:  CompIRF_ir.m
%%% Usage:  This program computes the IRFs of BHK model with an interest-rate rule. 
%%%         Producers face information stickiness.
%%%         Households have habit formation on nondurable consumption and 
%%%         incur adjustment cost on residential investment.
%%% Date:   January, 2021 
%%% Author: T. Takamura    
%%% Note:   Calls LINLAGEX by Meyer-Gohde
%%%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Set Parameter Values %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phiC    = paramvec.phiC;        % information stickiness on nondurable goods
phiX    = paramvec.phiX;        % information stickiness on durable goods 
varrho  = paramvec.varrho;		% sticky informtion on households 
thetaC  = paramvec.thetaC;      % price stickiness on nondurable goods
thetaX  = paramvec.thetaX;      % price stickiness on durable goods
beta    = paramvec.beta;		% discount factor
alpha   = paramvec.alpha;		% capital share
eta     = paramvec.eta;		    % Frisch elasticity of labor supply
epsilon = paramvec.epsilon;		% elasticity of demand
gamma   = paramvec.gamma;		% intra-temporal elasticity of substitution
sigma   = paramvec.sigma;		% inter-temporal elasticity of substitution
xi      = paramvec.xi;		    % curvature of housing adjustment cost function;
rhoI_1  = paramvec.rhoI_1;      % interest-rate coefficients of Taylor rule
rhoI_2  = paramvec.rhoI_2;
rhoPI   = paramvec.rhoPI;       % Response of interest rate to inflation
rhoY    = paramvec.rhoY; 	    % Response of interest rate to output 
rhoGY   = paramvec.rhoGY;       % Response of interest rate to output growth
kappaC  = paramvec.kappaC;      % Share of nondurable consumption in GDP
delta   = paramvec.delta;       % Depreciation rate of housing

omeg   = (1/eta+alpha)/(1-alpha);	% elasticity of individual marginal cost w.r.t. its own output
inv_gamma = 1.0/gamma;
inv_sigma = 1.0/sigma;
kappaX = 1-kappaC;

CHI     = (1.0-beta*(1.0-delta))/delta * (1.0-kappaC)/kappaC;
ucc     = (inv_gamma - inv_sigma)/(1.0 + CHI) - inv_gamma;
ucd     = (inv_gamma - inv_sigma)/(1.0 + 1.0/CHI);
udc     = ucc+inv_gamma;
udd     = ucd-inv_gamma;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  Output Settings    %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ntr           = 70;          % truncation number for lagged expectations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   Number and Indices of Variables  %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----- NUMBER OF ENDOGENOUS VARIABLES -----%
nvar  = 22;   
nz = 1;       % number of exogenous variables, e(t)
ne = 1;       % number of shocks

EXOGENOUS_VARIABLE_NAMES={'Policy rate shock'};

%----- VARIABLE NAMES -----%
ENDOGENOUS_VARIABLE_NAMES=...
{'pc'
'px'
'qc'
'qx'
'c'
'd'
'pcst'
'pxst'
'x'
'y'
'mc_c'
'mc_x'
'p'
'int'
'uc'
'ud'
'lam'
'cnet'
'mu'
'pic'
'pix'
'Lint'};


ipc    = 1;
ipx    = 2;
iqc    = 3;
iqx    = 4;
ic     = 5;
id     = 6;
ipc_st = 7;
ipx_st = 8;
ix     = 9;
iy     = 10;
imc_c  = 11;
imc_x  = 12;
ip     = 13;
iint   = 14;
iuc    = 15;
iud    = 16;
ilam   = 17;
icnet  = 18;
imu    = 19;
ipic   = 20;
ipix   = 21;
iLint  = 22;
        

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Initialize matrices %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
A_0      = zeros(nvar,nvar);
B_0      = zeros(nvar,nvar);
C_0      = zeros(nvar,nvar);
F_0      = zeros(nvar,nz);
G_0      = zeros(nvar,nz);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Construct the matrices %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
%(1) Euler Equation for savings 
B_0(1,ilam) = -1.0; 

% (2) Euler Equation for Durables Consumption 
B_0(2,imu)  = -1.0; 
B_0(2,iud)  =  (1.0-beta*(1.0-delta));

% (3) Definition of net consumption 
B_0(3,ic)   =  1.0; 
B_0(3,icnet)= -(1.0-varrho); 
C_0(3,ic)   = -varrho;
    
% (4) Consumption smoothing due to habit formation (connecting lambda and uc)
B_0(4,iuc)  = -1.0; 
B_0(4,ipc)  =  1.0-beta*varrho; 
B_0(4,ilam) =  1.0-beta*varrho; 

% (5) Law of Motion of Aggregate Durable Goods
B_0(5,id) = -1.0; 
B_0(5,ix) =  delta;
C_0(5,id) =  1.0-delta; 

% (6) Law of Motion of Nondurable-Goods Prices   
B_0(6,ipc)  = -1.0; 
B_0(6,iqc)  =  1.0-thetaC;
C_0(6,ipc)  =  thetaC; 
    
% (7) Law of Motion of Durable-Goods Prices
B_0(7,ipx) = -1.0; 
B_0(7,iqx) =  1.0-thetaX;
C_0(7,ipx) =  thetaX; 

% (8) Law of Motion of Optimized Nondurable-Good Prices 
B_0(8,iqc)    = -1.0; 
B_0(8,ipc_st) =  1.0-phiC; 
    
% (9) Law of Motion of Optimized Durable-Good Prices 
B_0(9,iqx)    = -1.0; 
B_0(9,ipx_st) =  1.0-phiX;

% (10) Pricing Euler Equation for Nondurable Goods  
B_0(10,ipc_st) =  1.0; 	

% (11) Pricing Euler Equation for Durable Goods 
B_0(11,ipx_st) =  1.0; 	

% (12) Marginal Cost Function (nondurable goods)
B_0(12,ilam)  = -1.0/(1.0+epsilon*omeg); 
B_0(12,ipc)   =  epsilon*omeg/(1.0+epsilon*omeg); 
B_0(12,ic)    =  omeg/(1.0+epsilon*omeg); 
B_0(12,imc_c) = -1.0;

% (13) Marginal Cost Function (durable goods)
B_0(13,ilam)  = -1.0/(1.0+epsilon*omeg);
B_0(13,ipx)   =  epsilon*omeg/(1.0+epsilon*omeg); 
B_0(13,ix)    =  omeg/(1.0+epsilon*omeg); 
B_0(13,imc_x) = -1.0;

% (14) Definition of Real GDP 
B_0(14,ic) =  kappaC; 
B_0(14,ix) =  kappaX; 
B_0(14,iy) = -1.0;
    
% (15) Definition of Aggregate Price Level
B_0(15,ip)  = -1.0; 
B_0(15,ipc) =  kappaC; 
B_0(15,ipx) =  kappaX;

% (16) Investment smoothing decision of HHs
B_0(16,imu) =  1.0;
B_0(16,ipx) = -1.0;
B_0(16,ilam)= -1.0;
B_0(16,ix)  = -(1.0+beta)*xi;
C_0(16,ix)  =  xi;

% (17) Definition of uc
B_0(17, iuc)   = -1.0;
B_0(17, icnet) =  ucc; 
B_0(17, id)    =  ucd; 

% (18) Definition of ud 
B_0(18, iud)   = -1.0;
B_0(18, icnet) =  udc; 
B_0(18, id)    =  udd; 

% (19) Definition of pic
B_0(19, ipic) =  -1.0;
B_0(19, ipc)  =   1.0;
C_0(19, ipc)  =  -1.0;

% (20) Definition of pix
B_0(20, ipix) =  -1.0;
B_0(20, ipx)  =   1.0;
C_0(20, ipx)  =  -1.0; 

% (21) Interest-Rate Rule
B_0(21, iint) = -1.0; 

B_0(21,ip)    =  (1.0-rhoI_1-rhoI_2)*rhoPI; 
B_0(21,iy)    =  (1.0-rhoI_1-rhoI_2)*(rhoY+rhoGY);
C_0(21,iy)    = -(1.0-rhoI_1-rhoI_2)*rhoGY; 
C_0(21,iint)  =  rhoI_1; 
C_0(21,iLint) =  rhoI_2;
C_0(21,ip)    = -(1.0-rhoI_1-rhoI_2)*rhoPI;

G_0(21,1)     =  1.0;

% (22) Definition of lagged policy rate
B_0(22, iLint) =  1.0;
C_0(22, iint)  = -1.0;

% POLICY RATE SHOCKS ARE i.i.d. 
N = 0.0;   
Omega = 1.0;

%name of the counter used in subsequent matrices
it_name='J_1';
%maximum value of counter: either 'infinity' or a scalar (without quotation marks)
it_max_value = Ntr;

%------------------------------------%
% Input elements in counter matrices %
%------------------------------------%

A_j=['[zeros(nvar,nvar)]'...
    '+(J_1==1)*[zeros(1,ilam-1) 1.0 zeros(1,nvar-ilam);'...
    'zeros(1,imu-1) beta*(1.0-delta) zeros(1, nvar-imu);'...
    'zeros(1,nvar);'...
    'zeros(1,iuc-1) beta*varrho zeros(1, nvar-iuc);'... 
    'zeros(5,nvar);'...
    'zeros(1,ipc_st-1) -beta*thetaC zeros(1,nvar-ipc_st);'...
    'zeros(1,ipx_st-1) -beta*thetaX zeros(1,nvar-ipx_st);'...
    'zeros(4,nvar);'...
    'zeros(1,ix-1) beta*xi zeros(1,nvar-ix);'...
    'zeros(6,nvar)]'];
B_j=['[zeros(nvar,nvar)]'...
	'+(J_1==1)*[zeros(1,iint-1) 1.0 zeros(1,nvar-iint);'...
    'zeros(8,nvar);'...
    'zeros(1,imc_c-1) -(1.0-thetaC*beta) zeros(1,nvar-imc_c);'...
    'zeros(1,imc_x-1) -(1.0-thetaX*beta) zeros(1,nvar-imc_x);'...
    'zeros(nvar-11,nvar)]'...
	'+(J_1>=2)*[zeros(7,nvar);'...
	'zeros(1, ipc_st-1) (1.0-phiC)*phiC^(J_1-1) zeros(1, nvar-ipc_st);'...
	'zeros(1, ipx_st-1) (1.0-phiX)*phiX^(J_1-1) zeros(1, nvar-ipx_st);'...
	'zeros(13,nvar)]'];
C_j='[zeros(nvar,nvar)]';
F_j='[zeros(nvar,nz)]';
G_j='[zeros(nvar,nz)]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%% SOLVE THE SYSTEM BY LINLAGEX %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PERIOD = 4;
RUN_IMPULSE = 0;
RUN_ANTICIPATED_IMPULSE=0;
RUN_SPECTRAL = 0;
RUN_SIMULATION = 0;
solution_method='AIM';

%---------------%
% Call LINLAGEX %
%---------------%
linlagex;

if (isnumeric(existence_uniqueness)==1)

	flag = 0;

	temp = -0.006/4.0*MA_VECTOR(:,1)*100;
	imp = reshape(temp, [nvar size(temp,1)/nvar])';

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%% Store the impulse responses %%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	pic = imp(hh,ipic);
	pix = imp(hh,ipix);
	c   = imp(hh,ic);
	x   = imp(hh,ix);

	IRF = [c; x; pic; pix];

else
	flag = 1;

	IRF = nan(length(hh), 4);

end
