function [IRF, flag] = CompIRF_mg(xparam,hh,scale)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                           %
%  Title: CompIRF_mg.m                                                      %
%  Usage: This file computes the dual-stickiness model with a               %
%         money-growth rule using the solution method of Meyer-Gohde (2007) %
%  Date: January 2021                                                       %
%  Author: Tamon Takamura                                                   %
%                                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parameters
beta    = xparam.beta;    % discount factor
delta   = xparam.delta;   % depreciation rate of durable goods
alpha   = xparam.alpha;   % capital share
eta     = xparam.eta;     % Frisch elasticity of labor supply
epsilon = xparam.epsilon; % elasticity of demand
kappaC  = xparam.kappaC;  % steady-state share of nondurable goods
thetaC  = xparam.thetaC;  % Calvo parameter for nondurable goods
thetaX  = xparam.thetaX;  % Calvo parameter for durable goods
phiC    = xparam.phiC;    % SI parameter for nondurable goods
phiX    = xparam.phiX;    % SI parameter for durable goods
rho_u   = xparam.gamma;   % Intra-temporal elasticity of substitution
sig_u   = xparam.sigma;   % Inter-temporal elasticity of substitution
rhoM    = xparam.rhoM;    % persistence of monetary policy shock

omegaw = 1/(eta*(1-alpha)); 
omegap = alpha/(1-alpha);   
omeg  = omegaw +omegap; 

rhoi   = 1.0/rho_u;
sigmai = 1.0/sig_u;

CHI     = (1.0-beta*(1.0-delta))/delta * (1.0-kappaC)/kappaC;
ucc     = (rhoi - sigmai)/(1.0 + CHI) - rhoi;
ucd     = (rhoi - sigmai)/(1.0 + 1.0/CHI);
udc     = ucc+rhoi;
udd     = ucd-rhoi;


%Variable names
ENDOGENOUS_VARIABLE_NAMES={'d'
'p'
'pc'
'px'
'qc'
'qx'
'pcst'
'pxst'
'mcc'
'mcx'
'y'
'c'
'x'
'int'
'uc'
'ud'
'gam'};

EXOGENOUS_VARIABLE_NAMES={'Money Supply1'};

id=1;
ip=2;
ipc=3;
ipx=4;
iqc=5;
iqx=6;
ipcst=7;
ipxst=8;
imcc=9;
imcx=10;
iy=11;
ic=12;
ix=13;
iint=14;
iuc=15;
iud=16;
igam = 17;

n=17;

A_0=zeros(n,n);
B_0=zeros(n,n);
C_0=zeros(n,n);
F_0=zeros(n,2);
G_0=zeros(n,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 0=sum A_iE_t-i(x_t+1)+sum B_iE_t-i(x_t)+sum C_iE_t-i(x_t-1)+sum   %
% F_iE_t-i(z_t+1)+sumG_iE_t-i(z_t)                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-----------------------------------%
% Input elements in A_0,....,G_0    %
%-----------------------------------%

%(1)0 = -(1.0-beta*(1.0-delta))*ud(t) + 1.0*gamma(t) -beta*(1.0-delta)*E_(t-1)[gamma(t+1)]
B_0(1,iud)  = -(1.0-beta*(1.0-delta)); 
B_0(1,igam) =  1.0;

%(2) 0 = 1.0*p(t) -kappaC*pc(t) -(1.0-kappaC)*px(t)
B_0(2,ip)  =  1.0; 
B_0(2,ipc) = -kappaC; 
B_0(2,ipx) = -(1.0-kappaC);

%(3) 0 = -thetaC*pc(t-1) + 1.0*pc(t) -(1-thetaC)*qc(t)
B_0(3,ipc) =  1.0; 
B_0(3,iqc) = -(1-thetaC);
C_0(3,ipc) = -thetaC; 

%(4) 0 = -thetaX*px(t-1) + 1.0*px(t) -(1-thetaX)*qx(t)
B_0(4,ipx) =  1.0; 
B_0(4,iqx) = -(1-thetaX);
C_0(4,ipx) = -thetaX;

%(5) 0 = 1.0*qc(t) -(1.0-phiC)*pcst(t) - (1.0-phiC)*sum(k=2 to Ntr) phiC^(k) E_(t-k)[pcst(t)]
B_0(5,iqc)   =  1.0; 
B_0(5,ipcst) = -(1.0-phiC);

%(6) 0 = 1.0*qx(t) - (1.0-phiX)*pxst(t) - (1.0-phiX)*sum(k=2 to Ntr) phiX^k E_(t-k)[pxst(t)]
B_0(6,iqx)   =  1.0; 
B_0(6,ipxst) = -(1.0-phiX);

%(7) 0 = -thetaC*beta*E_(t-1) pcst(t+1) + 1.0*pcst(t) - (1.0-thetaC*beta)* E_(t-1)mcc(t)
B_0(7,ipcst) =  1.0;

%(8) 0 = -thetaX*beta*E_(t-1) pxst(t+1) + 1.0*pxst(t) - (1.0-thetaX*beta)*E_(t-1) mcx(t)
B_0(8,ipxst) =  1.0;

%(9) 0 = -1.0*pc(t) + 1.0*mcc(t) + 1.0/(1.0+epsilon*omega)*uc(t) - omega/(1+epsilon*omega)*c(t)
B_0(9,ipc)  = -1.0; 
B_0(9,imcc) =  1.0; 
B_0(9,iuc)  =  1.0/(1.0+epsilon*omeg); 
B_0(9,ic)   = -omeg/(1.0+epsilon*omeg);

%(10) 0 = -epsilon*omega/(1.0+epsilon*omega)*px(t) - 1.0/(1.0+epsilon*omega)*pc(t) + 1.0*mcx(t) + 1.0/(1.0+epsilon*omega)*uc(t)
%         -omega/(1.0+epsilon*omega)*x(t)
B_0(10,ipc)  = -1.0/(1.0+epsilon*omeg);
B_0(10,ipx)  = -epsilon*omeg/(1.0+epsilon*omeg); 
B_0(10,imcx) =  1.0; 
B_0(10,iuc)  =  1.0/(1.0+epsilon*omeg);
B_0(10,ix)   = -omeg/(1.0+epsilon*omeg);

%(11) 0 = 1.0*p(t) + 1.0*y(t) - 1.0*m(t-1)
B_0(11,ip) =  1.0; 
B_0(11,iy) =  1.0; 
G_0(11,1)  = -1.0;

%(12) 0 = -1.0*y(t) + kappaC*c(t) + (1.0-kappaC)*x(t)
B_0(12,iy) = -1.0; 
B_0(12,ic) =  kappaC; 
B_0(12,ix) =  1.0-kappaC;

%(13) 0 = -1.0*d(t) + (1-delta)*d(t-1) + delta*x(t)
B_0(13,id) = -1.0; 
B_0(13,ix) =  delta;
C_0(13,id) =  1.0-delta; 

%(14) -1.0*E_(t-1) [pc(t+1) - uc(t+1)] +1.0*[pc(t)-uc(t)] + 1.0*R(t) = 0
B_0(14,ipc)  =  1.0;
B_0(14,iuc)  = -1.0;
B_0(14,iint) =  1.0; 

%(15) 0 = -1.0*uc(t) + ucc*c(t) + ucd*d(t)
B_0(15, iuc) = -1.0;
B_0(15, ic)  =  ucc;
B_0(15, id)  =  ucd;

% (16) 0 = -1.0*uc(t) + 1.0*gamma(t) - 1.0*px(t) + 1.0*pc(t) 
B_0(16, igam) =  1.0;
B_0(16, ipx)  = -1.0;
B_0(16, iuc)  = -1.0;
B_0(16, ipc)  =  1.0;

% (17) 0 = -1.0 * ud(t) + udc * c(t) + udd * d(t)
B_0(17, iud) = -1.0;
B_0(17, ic)  =  udc;
B_0(17, id)  =  udd;

%name of the counter used in subsequent matrices
it_name='J_1';
it_max_value=70;

%------------------------------------%
% Input elements in counter matrices %
%------------------------------------%

A_j=['[zeros(n,n)]'...
    '+(J_1==1)*[zeros(1,igam-1) -beta*(1.0-delta) zeros(1,n-igam);'...
    'zeros(5,n);'...
    'zeros(1,ipcst-1) -thetaC*beta zeros(1,n-ipcst);'...
    'zeros(1,ipxst-1) -thetaX*beta zeros(1,n-ipxst);'...
    'zeros(5,n);'...
    'zeros(1,ipc-1) -1.0 zeros(1,iuc-ipc-1) 1.0 zeros(1,n-iuc);'...
    'zeros(3,n)]'];
B_j=['[zeros(n,n)]'...
'+(J_1==1)*[zeros(6,n);'...
    'zeros(1,imcc-1) -(1.0-thetaC*beta) zeros(1,n-imcc);'...
    'zeros(1,imcx-1) -(1.0-thetaX*beta) zeros(1,n-imcx);'...
    'zeros(n-8,n)]'...
    '+(J_1>=2)*[zeros(4,n);'...
    'zeros(1,ipcst-1) -(1-phiC)*phiC^(J_1-1) zeros(1,n-ipcst);'...
    'zeros(1,ipxst-1) -(1-phiX)*phiX^(J_1-1) zeros(1,n-ipxst);'...
    'zeros(11,n)]'];
C_j='[zeros(n,n)]';
F_j='[zeros(n,2)]';
G_j='[zeros(n,2)]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Define exogenous process as a VAR(1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N = [    0,      1;
       -rhoM, 1+rhoM];
Omega= [0 0;
        0 1];

%Options
PERIOD=4;

RUN_IMPULSE=0;
RUN_ANTICIPATED_IMPULSE=0;
RUN_SPECTRAL=0;
RUN_SIMULATION=0;
solution_method = 'AIM';

linlagex;


if (isnumeric(existence_uniqueness)==1)

	flag = 0;

	temp=MA_VECTOR(:,2);
    imp=reshape(temp,[n size(temp,1)/n])';
    PiC=[imp(1,ipc);diff(imp(:,ipc))];
    PiX=[imp(1,ipx);diff(imp(:,ipx))];
    
    IRF=scale*[imp(hh,ic);imp(hh,ix);PiC(hh);PiX(hh)];

else
	flag = 1;

	IRF = nan(length(hh), 4);

end

end
