%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Title: mde_knitro_ir_noSI.m                                          %
% Usage: This program implements the minimum-distance estimation       % 
%        for the model with an interest-rate rule and no sticky info.  %
%        This program calls KNITRO's multi-start optimization routine. %
% Date: January 2021                                                   %
% Author: Tamon Takamura                                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;
clc;

% NUMBER OF PARAMETERS TO BE ESTIMATED
num_params= 3; 

% PARAMETERS
beta   =  0.995;
alpha  =  0.35;
delta  =  0.0125;
eta    =  1.0;
epsilon=  11.0;
thetaC =  0.50; % Price stickiness in nondurables
thetaX =  0.0;  % Price stickiness in residential investment
gamma  =  1.0;  % Intra-temporal elasticity of substitution between durable and nondurable goods
sigma  =  1.0;  % inter-temporal elasticity of substitution  
kappaC =  0.75;
xi     =  2.5;
varrho =  0.74; 

rhoI_1 =  1.05;
rhoI_2 = -0.13;
rhoPI  =  2.5; 
rhoY   =  0.11; 
rhoGY  =  1.50; 
phiC   =  0.0;  % no sticky information
phiX   =  0.0;  % no sticky information

xpara.thetaC = thetaC; 
xpara.thetaX = thetaX; 
xpara.beta   = beta; 
xpara.alpha  = alpha; 
xpara.eta    = eta; 
xpara.epsilon= epsilon; 
xpara.xi     = xi;
xpara.varrho = varrho;
xpara.gamma  = gamma;
xpara.sigma  = sigma;
xpara.kappaC = kappaC; 
xpara.delta  = delta; 
xpara.rhoI_1 = rhoI_1;
xpara.rhoI_2 = rhoI_2;
xpara.rhoPI  = rhoPI; 
xpara.rhoY   = rhoY; 
xpara.rhoGY  = rhoGY;
xpara.phiC   = phiC;
xpara.phiX   = phiX;

% EMPIRICAL IMPULSE RESPONSES AND ITS COVARIANCE MATRIX
load IRFV_empirical.mat;

tend = 25;
hh   = 2:tend;
nhh  = length(hh);

tic;

A = [];
b = [];
Aeq = [];
beq = [];
lb = zeros(num_params,1);
ub = [1;3;1];
nonlcon = [];
extendedFeatures = [];
options = [];
knitroOpts = [];
x0 = [0.78; 2.1; 0.75];
func = @(x) par_CompJ_ir_noSI(x, xpara, V, IRFhat, hh); 
[xminU,JU,exitflagU,outputU,lambdaU,gradU,hessianU] = ...
    knitromatlab(func,x0,A,b,Aeq,beq,lb,ub,nonlcon,extendedFeatures,options,'knitro.opt');

toc;


%---------------------------------------------%
% Compute the standard deviation of estimates %
%---------------------------------------------%
% Weighting matrix
IV = reshape(V, nhh+1, 4);
IV = IV(hh, 1:4);
IV = reshape(IV, nhh*4, 1); % Take out only non-zero elements
IV = 1.0 ./ IV;             % IV is an inverse of V

W = diag(IV);

% hstep is used to compute the first derivative of model-implied IRFs wrt a certain parameter
hstep = 1e-8;

xpara_mde  = xpara;
thetaX_mde = xminU(1);
xi_mde     = xminU(2);
varrho_mde = xminU(3); 

xpara_mde.thetaX   = thetaX_mde;
xpara_mde.xi       = xi_mde;
xpara_mde.varrho   = varrho_mde;

stdthetaX  = delta_method_ir(xpara_mde, hh, W, vcovmat, hstep, thetaX_mde,  'thetaX',  'CompIRF_ir');
stdxi      = delta_method_ir(xpara_mde, hh, W, vcovmat, hstep, xi_mde,      'xi',      'CompIRF_ir');
stdvarrho  = delta_method_ir(xpara_mde, hh, W, vcovmat, hstep, varrho_mde,  'varrho',  'CompIRF_ir');    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                             %
% PLOT MODEL AND EMPIRICAL RESPONSES TOGETHER %
%                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hor  = 1:tend;

IRFmodel = CompIRF_ir(xpara_mde, hor);
IRFmodel = reshape(IRFmodel,tend,4);

IRFhat = reshape(IRFhat, nstep, 4);
IRFc   = IRFhat(1:tend, 1);
IRFx   = IRFhat(1:tend, 2);
IRFpic = IRFhat(1:tend, 3);
IRFpix = IRFhat(1:tend, 4);

set(0,'DefaultLineLineWidth',2);
set(0,'DefaultAxesFontSize',16,'DefaultAxesFontName','Arial');
set(0,'DefaultTextFontSize',16,'DefaultTextFontName','Arial');

tt=0:tend-1;

fig1 = figure('PaperPosition',[0 0.14 11 8.5]);
orient landscape

subplot(2,2,1);
plot(tt,IRFc,'r',tt,IRFmodel(1:tend,1),'b-', tt,UCI(1:tend,1),'r--',tt,LCI(1:tend,1),'r--', tt, zeros(tend,1),'k:');
title('Nondurable goods and services')
ylabel('%');
legend('Empirical','Model');

xlim([0 tend-1]);

subplot(2,2,2);
plot(tt,IRFx,'r',tt,IRFmodel(1:tend,2),'b-',tt,UCI(1:tend,2),'r--',tt,LCI(1:tend,2),'r--', tt, zeros(tend,1),'k:');
title('Residential investment');
ylabel('%');
xlim([0 tend-1]);

subplot(2,2,3);
plot(tt,IRFpic,'r',tt,IRFmodel(1:tend,3),'b-',tt,UCI(1:tend,3),'r--',tt,LCI(1:tend,3),'r--', tt, zeros(tend,1),'k:');
title('Inflation of ND and services')
ylabel('%');
xlim([0 tend-1]);

subplot(2,2,4);
plot(tt,IRFpix,'r',tt,IRFmodel(1:tend,4),'b-',tt,UCI(1:tend,4),'r--',tt,LCI(1:tend,4),'r--', tt, zeros(tend,1),'k:');
title('Residential price inflation');
ylabel('%');
xlim([0 tend-1]);

%--------------%
% SAVE RESULTS %
%--------------%
save ResultKnitro_ir_noSI.mat IRFc IRFx IRFpic IRFpix IRFmodel UCI LCI xpara_mde stdthetaX stdxi stdvarrho JU xminU 
