%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Title: mde_knitro_mg.m                                               %
% Usage: This program implements the minimum distance estimation of    %
%        the model with a money-growth rule.                           %
%        This program calls KNITRO's multi-start optimization routine. %
% Date: January 2021                                                   %
% Author: Tamon Takamura                                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;
clc;

% NUMBER OF PARAMETERS TO BE ESTIMATED
num_params = 2;

% EMPIRICAL IMPULSE RESPONSES AND ITS COVARIANCE MATRIX
load IRFV_empirical.mat;

% PARAMETERS
beta   = 0.995;
alpha  = 0.35;
delta  = 0.0125;
eta    = 1.0;
epsilon= 11.0;
thetaC = 0.50;        % Price stickiness in nondurables
thetaX = 0.0;         % Price stickiness in residential investment
gamma  = 1.0;         % Intra-temporal elasticity of substitution between durable and nondurable goods
sigma  = 1.0;         % inter-temporal elasticity of substitution  
kappaC = 0.75;
rhoM   = ar1gmzm; 

xpara.thetaC = thetaC; 
xpara.thetaX = thetaX; 
xpara.beta   = beta; 
xpara.alpha  = alpha; 
xpara.eta    = eta; 
xpara.epsilon= epsilon; 
xpara.kappaC = kappaC;  
xpara.rhoM   = rhoM;
xpara.gamma  = gamma; 
xpara.sigma  = sigma;
xpara.delta  = delta;

tend = 25;
hh   = 2:tend;
nhh  = length(hh);

scale = mshock_ini;

tic;

% Baseline estimation 
A = [];
b = [];
Aeq = [];
beq = [];
lb = zeros(num_params,1);
ub = ones(num_params,1);
nonlcon = [];
extendedFeatures = [];
options = [];
knitroOpts = [];
x0 = [0.49; 0.74];
func = @(x) par_CompJ_mg(x, xpara, V, IRFhat, hh, scale); 
[xminU,JU,exitflagU,outputU,lambdaU,gradU,hessianU] = ...
    knitromatlab(func,x0,A,b,Aeq,beq,lb,ub,nonlcon,extendedFeatures,options,'knitro.opt');

toc;

% Restricted estimation: phiC=phiX
tic;
lb = zeros(num_params-1,1);
ub = ones(num_params-1,1);
x0 = 0.5*sum(xminU(1:2));
func3dimR = @(x) par_CompJ_mg_restricted(x, xpara, V, IRFhat, hh, scale);
[xminR,JR,exitflagR,outputR,lambdaR,gradR,hessianR] = ...
    knitromatlab(func3dimR,x0,A,b,Aeq,beq,lb,ub,nonlcon,extendedFeatures,options,'knitro.opt');
toc;

% testing the hypothesis of phiC = phiX
nR = 1;
testStat = JR - JU;
critval = chi2inv(0.95,nR);
fprintf('--------------------------------- \n')
formatSpec = 'J-statistic: %8.3e, critical value: %8.3e \n';
fprintf(formatSpec,testStat, critval);
if (testStat > critval)
    fprintf('The null hypothesis of phiC=phiX is rejected. \n')
else
    fprintf('The null hypothesis of phiC=phiX cannot be rejected. \n')
end
fprintf('--------------------------------- \n')


phiC_mde    = xminU(1);
phiX_mde    = xminU(2);


%---------------------------------------------%
% Compute the standard deviation of estimates %
%---------------------------------------------%
% Weighting matrix
IV = reshape(V, 25, 4);
IV = IV(hh, 1:4);
IV = reshape(IV, nhh*4, 1); % Take out only non-zero elements
IV = 1.0 ./ IV;             % IV is an inverse of V

W = diag(IV);

% hstep is used to compute the first derivative of model-implied IRFs wrt a certain parameter
hstep = 1e-8;

xpara_mde      = xpara;
xpara_mde.phiC = phiC_mde;
xpara_mde.phiX = phiX_mde;

stdphiC    = delta_method_mg(xpara_mde, hh, W, vcovmat, hstep, scale, phiC_mde,    'phiC',   'CompIRF_mg');
stdphiX    = delta_method_mg(xpara_mde, hh, W, vcovmat, hstep, scale, phiX_mde,    'phiX',   'CompIRF_mg');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                             %
% PLOT MODEL AND EMPIRICAL RESPONSES TOGETHER %
%                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hor  = 1:tend;

IRFmodel = CompIRF_mg(xpara_mde, hor, scale);
IRFmodel = reshape(IRFmodel,tend,4);

IRFhat = reshape(IRFhat, nstep, 4);
IRFc   = IRFhat(1:tend, 1);
IRFx   = IRFhat(1:tend, 2);
IRFpic = IRFhat(1:tend, 3);
IRFpix = IRFhat(1:tend, 4);

set(0,'DefaultLineLineWidth',2);
set(0,'DefaultAxesFontSize',16,'DefaultAxesFontName','Arial');
set(0,'DefaultTextFontSize',16,'DefaultTextFontName','Arial');

tt=0:tend-1;

fig1 = figure('PaperPosition',[0 0.14 11 8.5]);
orient landscape

subplot(2,2,1);
plot(tt,IRFc,'r',tt,IRFmodel(1:tend,1),'b-', tt,UCI(1:tend,1),'r--',tt,LCI(1:tend,1),'r--', tt, zeros(tend,1),'k:');
title('Nondurable goods and services')
ylabel('%');
legend('Empirical','Model');

xlim([0 tend-1]);

subplot(2,2,2);
plot(tt,IRFx,'r',tt,IRFmodel(1:tend,2),'b-',tt,UCI(1:tend,2),'r--',tt,LCI(1:tend,2),'r--', tt, zeros(tend,1),'k:');
title('Residential investment');
ylabel('%');
xlim([0 tend-1]);

subplot(2,2,3);
plot(tt,IRFpic,'r',tt,IRFmodel(1:tend,3),'b-',tt,UCI(1:tend,3),'r--',tt,LCI(1:tend,3),'r--', tt, zeros(tend,1),'k:');
title('Inflation of ND and services')
ylabel('%');
xlim([0 tend-1]);

subplot(2,2,4);
plot(tt,IRFpix,'r',tt,IRFmodel(1:tend,4),'b-',tt,UCI(1:tend,4),'r--',tt,LCI(1:tend,4),'r--', tt, zeros(tend,1),'k:');
title('Residential price inflation');
ylabel('%');
xlim([0 tend-1]);

%--------------%
% SAVE RESULTS %
%--------------%

save ResultKnitro_mg.mat IRFc IRFx IRFpic IRFpix IRFmodel UCI LCI xpara_mde stdphiC stdphiX xminU xminR testStat critval JU JR

