function [varimpneg,varimppos,varz,cimat]=mkimpci(beta,a0inv,nlags,errshk,nstep,ndraws,nobs,pctg)

% MKIMPCI calculates confidence intervals for impulse response functions.
% VARIMPNEG - Estimated Impulse - Norminv(1-pctg,0,1)*STD Dev 
% VARIMPPOS - Estimated Impulse + Norminv(1-pctg,0,1)*Std Dev 
% VARZ = Sample Variance
% Creates Monte Carlo confidence intervals 
% Generates 2*nobs observations of Data 
% using Y = beta*Ylag + a0inv*ut where ut is i.i.d standard Normal
% Estimates a var on the last nobs observations 
% Uses the VAR estimate to generate impulse responses 
% Repeats ndraws times
% Calculates the sample variance for each point estimate.
% Returns Impvar which is the estimated impulse response function + or - Norminv(1-pctg,0,1)*standard Errors

% NOTE The starting values are always set equal to zero. I however
% throw away the first nobs of data to reduce the dependence on starting values.

% UPDATED BY T. TAKAMURA IN jANUARY 2009.
% (1) SET A seed=2009 FOR randn()
% (2) EXPANDED THE OUTPUT LIST TO INCLUDE cimat
% (3) TAKES A0inv AS INPUT. NO NEED TO TAKE THE INVERSE OF A0.

beta = beta';
nvars = length(a0inv);
hascon = size(beta,1)-nvars*nlags;

ydata = zeros(2*nobs,nvars);
erz = zeros(2*nobs,nvars);

randn('seed',2009);

for icnt=1:ndraws

    disp(['Now doing trial ' num2str(icnt)])
    %loop over the number of draws
    % generate data and estimate the var
    % and then calculate impulse response functions
   
    % monte carlo method to generate the data
    u=randn(2*nobs,nvars); % create the fundamental error terms N(0,1)
    for jj=1:2*nobs
     erz(jj,:)=(a0inv*u(jj,:)')';
     %create the VAR error terms       
    end
   
    %we will add in the constant term to the residual
    %it makes the calculations easier ahead.
    for kk=1:nvars
  	    erz(:,kk)=erz(:,kk)+hascon*beta(1,kk);
        %create the VAR error terms plus a constant if included
    end
   
    % intialize the first observations
    zlag=[];
    for k=1:nlags
       ydata(k,:)=erz(k,:);
       zlag = [ydata(k,:) zlag]; 
    end
   
    for jj=nlags+1:2*nobs
        ydata(jj,:)=erz(jj,:)+zlag*beta(hascon+1:size(beta,1),:);
        zlag=[ydata(jj,:) zlag(1,1:(nlags-1)*nvars)];
    end
   
    %estimate var and calculate a0 matrix on simulated data
    %I throw away the first nobs of data as I want to avoid any starting value problems
    [best,sigest]=estimatevar(ydata(1*nobs+1:end,:),nlags,hascon);
   
    a0estinv=chol(sigest)';
      
    %calculate impulse responses
    impz=mkimprep(best,a0estinv,nlags,errshk,nstep);
    %if you don't have three dimensional arrays this will break.
    cnfdints(:,:,icnt)=impz;
    %save them and do it again   

end

cimat=cnfdints;

%find the upper and lower bounds. 
cnfsrt= sort(cnfdints,3);

for zx=1:nvars
   for zy=1:nstep
      cnfdints(zy,zx,:) = cnfdints(zy,zx,:)-mean(cnfdints(zy,zx,:),3);
   end
end

% Mean responses around which confidence intervals are constructed.
trimpz=mkimprep(beta',a0inv,nlags,errshk,nstep);

% Variances
varz = (1/ndraws)*sum(cnfdints.^2,3);

%norminv is the inverse of the normal cdf function. If you don't have the stats toolbox this line won't work. 
varimppos = trimpz + norminv(1-pctg,0,1)*sqrt(varz);
varimpneg = trimpz - norminv(1-pctg,0,1)*sqrt(varz);


