function vcovmat = mkvcov(cimat, indy, indp, ndraws, nstep, nvar)

% TITLE: mkvcov.m
%
% USAGE: THIS FUNCTION COMPUTES A VARIANCE-COVARIANCE MATRIX OF EMPIRICAL
%        IMPULSE RESPONSES FOR SELECTED VARIABLES. THE VARIANCE-COVARIANCE
%        MATRIX THAT THIS FUNCTION COMPUTES EXCLUDES RESPONSES IN PERIOD 0 
%        AS BOTH THE MODEL AND SVAR IMPOSE A RESTRICTION THAT THEY ARE ZERO.
%
% INPUT: cimat: A (nstep x numvars x ndraws) MATRIX CONTAINING IMPULSE RESPONSES TO
%               SHOCKS GENERATED BY A MONTE-CARLO SIMULATION. numvars IS
%               THE TOTAL NUMBER OF VARIABLES IN THE SVAR.
%        indy: A (1 x ny) VECTOR CONTAINING INDICES OF REAL VARIABLES TO BE
%              INCLUDED IN vcovmat. 
%        indp: A (1 x np) VECTOR CONTAINING INDICES OF PRICE VARIABLES TO BE
%              INCLUDED IN vcovmat. 
%        ndraws: # DRAWS IN A MONTE-CARLO SIMULATION
%        nsteps: # PERIODS FOR WHICH IMPULSE RESPONSES ARE CONSIDERED
%        nvar:   # VARIABLES FOR WHICH THE VARIANCE-CONVARIANCE MATRIX IS COMPUTED.
%
% Output: vcovmat: AN (nvars*nstep x nvars*nstep) MATRIX OF EMPIRICAL
%                  IMPULSE RESPONSES.
%
% DATE: JANUARY 21, 2014
% AUTHOR: TAMON TAKAMURA

[n1, n2, n3] = size(cimat);

if (n1~=nstep)
    string = ['size(cimat,1) must be ', num2str(nstep)];
    error(string);
elseif (n3~=ndraws)
    string = ['size(cimat,3) must be ', num2str(ndraws)];
    error(string);
end

zmat = cimat(2:end, indy, :);

infmat = cimat(2:end, indp, :) - cimat(1:end-1, indp, :);

zmat = [zmat, infmat];

thetavec = reshape(zmat, (nstep-1)*nvar, 1, ndraws);

mean_theta = mean(thetavec, 3);

y = zeros((nstep-1)*nvar, (nstep-1)*nvar);

for in = 1:ndraws
    
    x = squeeze(thetavec(:, :, in)) - mean_theta;
    y = y + x*x';
    
end

vcovmat = y / ndraws;
