%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%  TITLE:  plot_fig1.m                                                    %
%  DATE:   JANUARY 2021                                                   %
%  AUTHOR: Tamon Takamura                                                 %
%  USAGE:  THIS PROGRAM COMPUTES IMPULSE RESPONSE FUNCTIONS TO A          %
%          MONETARY POLICY SHOCK IDENTIFIED BY THE BLOCK-RECURSIVE        %
%          ASSUMPTION IN VAR.                                             %
%                                                                         %
%  NOTE:   THIS PROGRAM CALLS VAR SUBROUTINES WRITTEN BY R. VIGFUSSON     %
%                                                                         %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;
clc;

%%%%%%%%%%%%%%%%%%%%%%
%       DATA         %
%%%%%%%%%%%%%%%%%%%%%%

% READ DATA
load VAR_data.mat; 

rsd     = data0(:,1);
dur     = data0(:,2);
ndsv    = data0(:,3);
pr_rsd  = data0(:,4);
pr_dur  = data0(:,5);
pr_ndsv = data0(:,6);
pcom    = data0(:,7);
ffr     = data0(:,8);
gmzm    = data0(:,9);

% ANNUAL RATE => QUARTERLY RATE
ffr=ffr/400;  

% LOG OF OUTPUT SERIES
lrsd    =   log(rsd);
ldur    =   log(dur);
lndsv   =   log(ndsv);

% LOG OF PRICE SERIES
lp_rsd    =   log(pr_rsd);
lp_dur    =   log(pr_dur);
lpcom     =   log(pcom);
lp_ndsv   =   log(pr_ndsv);

%%%%%%%%%%%%%%%%%%%
%      SVAR       %
%%%%%%%%%%%%%%%%%%%
nlags=4;     % # OF VAR LAGS
hascon=1;    % INCLUDE A CONSTANT IN VAR

data = [lrsd, ldur, lndsv, lp_rsd, lp_dur, lp_ndsv, lpcom, ffr, gmzm];

irsd    = 1;
idur    = 2;
indsv   = 3;
ip_rsd  = 4;
ip_dur  = 5;
ip_ndsv = 6;
ipcom   = 7;
iffr    = 8;
im      = 9;

errshk=8;    

indy = [indsv, irsd];
indp = [ip_ndsv, ip_rsd];

% ESTIMATE REDUCED-FORM VAR
[betaz,sigma,residuals]=estimatevar(data,nlags,hascon);

% INVERSE OF ROTATION MATRIX
A0inv=chol(sigma)';

% # OF STEPS TO SIMULATE
nstep=25;

% # OF VARIABLES TO BE INCLUDED IN MDE 
nvar = 4;

% IMPULSE RESPONSE FUNCTION BASED ON THE VAR ESTIMATE ABOVE
impzmat=mkimprep(betaz,A0inv,nlags,errshk,nstep);

% MONTE CARLO METHOD TO CONSTRUCT CONFIDENCE INTERVALS
pctg=0.025;         % 95% CONFIDENCE INTERVAL
ndraws=500;         % # OF REPETITIONS FOR MONTE CARLO
nobs=size(data,1);  % # OF OBSERVATIONS IN EACH MONTE CARLO SIMULATION

[cilasy, ciuasy, varci, cimat]=mkimpci(betaz,A0inv,nlags,errshk,nstep,ndraws,nobs,pctg);

% CREATE EACH IMPULSE RESPONSE SERIES AND THEIR CONFIDENCE INTERVALS
imp_dur     = impzmat(:,idur);
imp_pdur    = impzmat(:,ip_dur);
imp_infdur  = [0;diff(imp_pdur)];

imp_rsd     = impzmat(:,irsd);
imp_ndsv    = impzmat(:,indsv);
imp_prsd    = impzmat(:,ip_rsd);
imp_pndsv   = impzmat(:,ip_ndsv);
imp_ffr     = impzmat(:,iffr);
imp_infrsd  = [0;diff(imp_prsd)];
imp_infndsv = [0;diff(imp_pndsv)];
imp_m       = impzmat(:,im);

% CONFIDENCE INTERVALS FOR INFLATION
[cilasy_infdur,ciuasy_infdur,varci_inf_dur]=mkinfci(cimat,ip_dur,pctg,ndraws,nstep,imp_infdur);
[cilasy_infrsd,ciuasy_infrsd,varci_inf_rsd]=mkinfci(cimat,ip_rsd,pctg,ndraws,nstep,imp_infrsd);
[cilasy_infndsv,ciuasy_infndsv,varci_inf_ndsv]=mkinfci(cimat,ip_ndsv,pctg,ndraws,nstep,imp_infndsv);

% CONFIDENCE INTERVALS FOR OTHER VARIABLES
cil_dur  = cilasy(:,idur);
cil_rsd  = cilasy(:,irsd);
cil_ndsv = cilasy(:,indsv);
cil_ffr  = cilasy(:,iffr);
cil_m    = cilasy(:,im);
    
ciu_dur  = ciuasy(:,idur);
ciu_rsd  = ciuasy(:,irsd);
ciu_ndsv = ciuasy(:,indsv);
ciu_ffr  = ciuasy(:,iffr);   
ciu_m    = ciuasy(:,im);
    
% VARIANCE-COVARIANCE MATRIX FOR VARIABLES OF INTEREST
vcovmat = mkvcov(cimat, indy, indp, ndraws, nstep, nvar);
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Plots                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
set(0,'DefaultLineLineWidth',2);
set(0,'DefaultAxesFontSize',10,'DefaultAxesFontName','Arial');
set(0,'DefaultTextFontSize',15,'DefaultTextFontName','Arial');


t=0:nstep-1;

figure1 = figure('WindowState','maximized');
set(figure1,'PaperOrientation','landscape')
set(figure1, 'PaperPosition', [0 0 11 8.5]) 

subplot(2,3,1)
plot(t,-imp_dur*100,'b', 'LineWidth', 3)
hold on
plot(t,-cil_dur*100,'--r',t,-ciu_dur*100,'--r',t,zeros(nstep,1),'k')
hold off
title('Durable Goods','FontSize',12,'Interpreter','Latex')
xlim([0 nstep-1])

subplot(2,3,2)
plot(t,-imp_rsd*100,'b', 'LineWidth', 3)
hold on
plot(t,-cil_rsd*100,'--r',t,-ciu_rsd*100,'--r',t,zeros(nstep,1),'k')
hold off
title('Residential Investment','FontSize',12,'Interpreter','Latex')
xlim([0 nstep-1])

subplot(2,3,3)
plot(t,-imp_ndsv*100,'b', 'LineWidth', 3)
hold on
plot(t,-cil_ndsv*100,'--r',t,-ciu_ndsv*100,'--r',t,zeros(nstep,1),'k')
hold off
title('Nondurable Goods \& Services','FontSize',12,'Interpreter','Latex')
xlim([0 nstep-1])

subplot(2,3,4)
plot(t,-imp_infdur*100,'b', 'LineWidth', 3)
hold on
plot(t,-cilasy_infdur*100,'--r',t,-ciuasy_infdur*100,'--r',t,zeros(nstep,1),'k')
hold off
title('Durable Goods Inflation','FontSize',12,'Interpreter','Latex')
xlim([0 nstep-1])

subplot(2,3,5);
plot(t,-imp_infrsd*100,'b', 'LineWidth', 3)
hold on
plot(t,-cilasy_infrsd*100,'--r',t,-ciuasy_infrsd*100,'--r',t,zeros(nstep,1),'k')
hold off
title('Residential Price Inflation','FontSize',12,'Interpreter','Latex')
xlim([0 nstep-1])

subplot(2,3,6)
plot(t,-imp_infndsv*100,'b', 'LineWidth', 3)
hold on
plot(t,-cilasy_infndsv*100,'--r',t,-ciuasy_infndsv*100,'--r',t,zeros(nstep,1),'k')
hold off
title('Nondurable Goods \& Services Inflation','FontSize',12,'Interpreter','Latex')
xlim([0 nstep-1])

y_gmzm  = gmzm;
x_gmzm  = [ones(nobs,1), [0.00191656610126234; gmzm(1:end-1)]];
b_gmzm  = (x_gmzm'*x_gmzm)\(x_gmzm'*y_gmzm);
ar1gmzm = b_gmzm(2);

IRFhat      = -100.0   * [imp_ndsv; imp_rsd; imp_infndsv; imp_infrsd];
V           =  10000.0 * [varci(:,indsv); varci(:,irsd);varci_inf_ndsv; varci_inf_rsd];
UCI         = -100.0   * [ciu_ndsv, ciu_rsd, ciuasy_infndsv, ciuasy_infrsd];
LCI         = -100.0   * [cil_ndsv, cil_rsd, cilasy_infndsv, cilasy_infrsd];
mshock_ini  = -100.0   * imp_m(1);
vcovmat     =  10000.0 * vcovmat;

save('IRFV_empirical.mat', 'cimat', 'IRFhat', 'V', 'vcovmat', 'UCI', 'LCI', 'mshock_ini',...
'indsv', 'irsd', 'ip_ndsv', 'ip_rsd','ar1gmzm', 'ndraws', 'nstep');  
    


