%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%     Matlab code to compute the impulse responses of
%%%     BHK model with sticky information (alternative labor mobility specification) 
%%%
%%%     written by Tamon Takamura
%%%     JANUARY 2021
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
clc
tic

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Set parameter values %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
beta    = 0.98^0.25;  % discount factor
alpha   = 0.35;       % capital share
delta   = 0.05/4;     % depreciation rate of durable goods
eta     = 1.0;        % Frisch elasticity of labor supply
epsilon = 11.0;       % elasticity of demand
theta   = 0.5;        % sticky price parameter (Calvo prob) 
impT    = 26;         % number of periods to calculate the impulse responses
xi      = 2.0;        % parameter value determining the labor elasticity of substitution across sectors: consistent with Bouakez et al. (2011) 

% PARAMETER VALUES TO SIMULATE
% order: [kappaC,    phiC,   phiX,   thetaC, thetaX, sigma, rho,  delta,    eta, alpha, beta, epsilon, xi]
paramvec=[  0.75,    0.0,    0.0,    theta,    0.0,   1.0,   1.0,  delta,    eta, alpha, beta, epsilon, xi; 
            0.75,    0.25,   0.25,   theta,    0.0,   1.0,   1.0,  delta,    eta, alpha, beta, epsilon, xi;  
            0.75,    0.5,    0.5,    theta,    0.0,   1.0,   1.0,  delta,    eta, alpha, beta, epsilon, xi; 
            0.75,    0.75,   0.75,   theta,    0.0,   1.0,   1.0,  delta,    eta, alpha, beta, epsilon, xi];  
   
    
[p, pi, pc, pic, px, pix, y, c, x] = dual_stickiness_Mobility(paramvec, impT);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  Plot the impulse response functions %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lw = 2;
set(0,'DefaultLineLineWidth',lw)
set(0,'DefaultAxesFontSize',20,'DefaultAxesFontName','Arial')
set(0,'DefaultTextFontSize',20,'DefaultTextFontName','Arial')
ltypes = char('r--','k:','b-*','b-'); % line types in graphs

figure('units','normalized','outerposition',[0 0 1 1])
orient portrait
set(gcf,'renderer','Painters')
set(gcf, 'Color', 'w');

subplot(2,3,1)
    for idx=1:size(paramvec,1)
        if (idx~=3)
            plot((0:(impT-1)), c(:,idx), deblank(ltypes(idx,:)))
        else
            plot((0:(impT-1)), c(:,idx), deblank(ltypes(idx,:)),'Color', [0.4 0.4 0.4],'LineWidth',3)
        end
        hold on
    end
    xlim([0 15])
    title('Nondurable Production','Interpreter','Latex','Fontsize',16)

    subplot(2,3,2)
    for idx=1:size(paramvec,1)
        if (idx~=3)
            plot((0:(impT-1)), x(:,idx), deblank(ltypes(idx,:)))
        else
            plot((0:(impT-1)), x(:,idx), deblank(ltypes(idx,:)),'Color', [0.4 0.4 0.4],'LineWidth',3)
        end
        hold on
    end
    xlim([0 15])
    title('Durable Production','Interpreter','Latex','Fontsize',16)

    subplot(2,3,3)
    for idx=1:size(paramvec,1)
        if (idx~=3)
            plot((0:(impT-1)), px(:,idx)-pc(:,idx), deblank(ltypes(idx,:)))
        else
            plot((0:(impT-1)), px(:,idx)-pc(:,idx), deblank(ltypes(idx,:)),'Color', [0.4 0.4 0.4],'LineWidth',3)
        end
        hold on
    end
    xlim([0 15])
    title('Relative Price','Interpreter','Latex','Fontsize',16)

    subplot(2,3,4)
    for idx=1:size(paramvec,1)
        if (idx~=3)
            plot((0:(impT-1)), pic(:,idx), deblank(ltypes(idx,:)))
        else
            plot((0:(impT-1)), pic(:,idx), deblank(ltypes(idx,:)),'Color', [0.4 0.4 0.4],'LineWidth',3)
        end
        hold on
    end
    xlim([0 15])
    title('Nondurable Inflation','Interpreter','Latex','Fontsize',16)

    subplot(2,3,5)
    for idx=1:size(paramvec,1)
        if (idx~=3)
            plot((0:(impT-1)), pix(:,idx), deblank(ltypes(idx,:)))
        else
            plot((0:(impT-1)), pix(:,idx), deblank(ltypes(idx,:)),'Color', [0.4 0.4 0.4],'LineWidth',3)
        end
        hold on
    end
    xlim([0 15])
    title('Durable Inflation','Interpreter','Latex','Fontsize',16)

    hSub = subplot(2,3,6); 
    for idx=1:size(paramvec,1)
        if (idx~=3)
            plot(1, nan, deblank(ltypes(idx,:))) 
        else
            plot(1, nan, deblank(ltypes(idx,:)),'Color', [0.4 0.4 0.4],'LineWidth',3) 
        end
        hold on
    end 
    set(hSub, 'Visible', 'off');

    legend_s1 = ['$ \phi_{c} = \phi_{x} = ',num2str(paramvec(1,2)),'$'];
    legend_s2 = ['$ \phi_{c} = \phi_{x} = ',num2str(paramvec(2,2)),'$'];
    legend_s3 = ['$ \phi_{c} = \phi_{x} = ',num2str(paramvec(3,2)),'$'];
    legend_s4 = ['$ \phi_{c} = \phi_{x} = ',num2str(paramvec(4,2)),'$'];
    legend(legend_s1, legend_s2, legend_s3, legend_s4, 'Interpreter','Latex','Fontsize',18);


toc
